import sys, os, time
from rlextra.ers.framework import WrappedXMLProcessor
class BrochureWrappedXMLProcessor(WrappedXMLProcessor):
    if sys.hexversion<0x2020000:
        def getActions(self):
            "return tuple of valid actions for this project"
            return WrappedXMLProcessor.getActions(self,plus=("_dtFilterPdf",))

    def action_dtFilterPdf(self,params):
        '''take filter=d01_02,... and sample params and get a filtered brochure'''
        from conftree import ConfTree
        ct = ConfTree(params['data'])
        F = params.get('filter','').split(',')
        I = ct.getItems()
        D = ct.getDays()
        T = ct.getTracks()
        tree = ct.tree
        OI = tree.conference.program_items._namedNodes('item')
        NI = []
        for i in I:
            d = D.index(i.day)
            t = T.index(i.track)
            if ('d%dt%d' % (d,t)) in F: NI.append(OI[I.index(i)])
        tree.conference.program_items._replaceNamedChildren('item',NI,all=1)
        from rlextra.radxml.xmlutils import reconstructXML
        for k in 'generatedBy','generatedFor':
            v = params.get(k,'')
            if v: tree.conference._setAttr(**{k: v})

        C = []
        V = ct.getInterviews()
        for c in tree.conference._namedChildren('text_page'):
            if str(c.category)=='interview':
                if ('i%d' % V.index(str(c.name).strip())) in F: C.append(c)
            else:
                C.append(c)
        tree.conference._replaceNamedChildren('text_page',C,all=1)

        params['data'] = reconstructXML(tree,1)
        self.actionPdf(params)

def go(params):
    if sys.hexversion>=0x2020000:
        import cgitb
        #cgitb.enable()
    return BrochureWrappedXMLProcessor().go(params)

def run(xmlFileName):
    "Here's one we made by hand"

    t0 = time.clock()
    outFileName = os.path.splitext(xmlFileName)[0] + '.pdf'
    D = {'outfile':outFileName}  


    filter = os.environ.get('RL_BROCHURE_FILTER',None)
    if not filter is None:
        D['filter'] = filter
        D['action'] = '_dtFilterPdf'
        D['data'] = xmlFileName
        D['generatedFor'] = 'Robin Becker'
    else:
        xmlText = open(xmlFileName).read()
        t1 = time.clock()
        print 'fetched %d bytes of xml in %0.4f seconds' % (len(xmlText), t1-t0)
        D['data'] = xmlText
        D['action'] = 'Pdf'

    t1 = time.clock()
    go(D)
    t2 = time.clock()
    print 'wrote %s in %0.4f seconds' % (outFileName, t2-t1)
    print 'total time = %0.4f seconds' % (t2-t0)
    
if __name__=='__main__':
    args = sys.argv[:]
    if len(args) < 2:
        print 'usage: brochure.py myfile.xml'
    else:
        run(args[1])
