import sys, os
from rlextra.radxml.xmlutils import reconstructXML

class Speaker:
    def __init__(self, id, name, email, company, url, bio, status):
        self.id = id
        self.name = name
        self.email = email
        self.company = company
        self.url = url
        self.bio = bio
        self.status = status

    def __str__(self):
        return 'id: %s\nname: %s\nemail: %s\ncompany: %s\nurl: %s\nbio: %d bytes\nstatus: %s\n' % (self.id,self.name,self.email,self.company,self.url,len(self.bio),self.status)

class Item:
    def __init__(self, day, start, duration, track, title,
                        desc, speaker, room, kind,
                        nr, status):
        self.day = day
        self.start = start
        self.duration = duration
        self.track = track
        self.title = title
        self.desc = desc
        self.speaker = speaker
        self.room = room
        self.kind = kind
        self.nr = nr
        self.status = status

    def __str__(self):
        return 'day: %s\nstart: %s\nduration: %s\ntrack: %s\ntitle: %s\ndesc: %s\nspeaker: %s\nroom: %s\nkind: %s\nnr: %s\nstatus: %s' % (self.day, self.start, self.duration, self.track, self.title, self.desc, self.speaker, self.room, self.kind, self.nr, self.status)

class ConfTree:
    def __init__(self,f):
        '''get access to the data defined by f(ile)/f(ilename)
        And create an xmltree of the data in the file
        '''
        if hasattr(f,'read'):
            xml = f.read()
        else:
            from reportlab.lib.utils import open_for_read
            xml = open_for_read(f).read()
        
        dtdDir = os.path.abspath(os.path.dirname(__name__=='__main__' and sys.argv[0] or __file__))
        cwd = os.getcwd()
        if dtdDir!=cwd: os.chdir(dtdDir)
        try:
            from rlextra.radxml.xmlutils import escapeNakedAmpersands, xml2doctree
            self.tree = xml2doctree(escapeNakedAmpersands(xml), validating=1)
        finally:
            if dtdDir!=cwd: os.chdir(cwd)

    def getItems(self):
        if not hasattr(self,'_items'):
            I = []
            a = I.append
            for i in self.tree.conference.program_items._namedChildren('item'):
                a(Item( str(i.day), str(i.start), str(i.duration), str(i.track), str(i.title),
                        reconstructXML(i.description), str(i.speaker.id), str(i.room), str(i.kind),
                        str(i.session_nr), str(i.status),
                        )
                    )
            self._items = I
        return self._items

    def getTracks(self):
        if not hasattr(self,'_tracks'):
            T = []
            a = T.append
            for i in self.getItems():
                t = str(i.track)
                if t not in T: a(t)
            T.sort()
            self._tracks = T
        return self._tracks

    def getDays(self):
        if not hasattr(self,'_days'):
            D = []
            a = D.append
            for i in self.getItems():
                d = str(i.day)
                if d not in D: a(d)
            D.sort()
            self._days = D
        return self._days

    def getInterviews(self):
        if not hasattr(self,'_interviews'):
            I = []
            for i in self.tree.conference._namedChildren('text_page'):
                if str(i.category).lower()=='interview':
                    I.append(str(i.name).strip())
            self._interviews = I
        return self._interviews

    def getSpeakerFromId(self,id):
        self.getSpeakers()
        return self._speakers[id]

    def getSpeakers(self):
        if not hasattr(self,'_speakers'):
            S = {}
            for i in self.tree.conference.speaker_bios._namedChildren('speaker_info'):
                id = str(i.id)
                S[str(id)] = Speaker(str(id), str(i.name), str(i.email), str(i.company),
                                        str(i.url), reconstructXML(i.biography), str(i.status))
            self._speakers = S
        return self._speakers

if __name__=='__main__':
    ct = ConfTree('sample_ukpython_2003.xml')
    print ct.getTracks()
    print ct.getDays()
    for i in ct.getSpeakers():
        print '\n',i
    for i in ct.getItems():
        print '\n',i
