def _incrNum(s):
	i = int(s)+1
	n = str(i)
	return (len(s)-len(n))*'0' + n

def _validInt(s,l,h):
	try:
		n = int(s)
		return l<=n<=h
	except:
		return 0

def _yearCmp(a,b):
	a0, b0 = a[0], b[0]
	if a0=='B' and b0=='B': return 0
	elif a0=='B': return 1
	elif b0=='B': return -1
	else: return cmp(int(b),int(a))

def _makeSeqPTag(contents,OUT):
	for t in contents:
		if type(t) is type(''):
			OUT.append(t)
		else:
			OUT.append(t.makePTag())
	return OUT

class PseudoTag:
	_contentNames = []
	_noGroupNames = ()

	def __init__(self,contents,**attr):
		for k in '_noGroupNames', '_contentNames', '_tagName':
			if attr.has_key(k):
				setattr(self,k,attr[k])
				del attr[k]
		self._contents = contents 
		self._attr = attr

	def __repr__(self):
		a = '%s(%s' % (self.__class__.__name__,repr(self._contents))
		for k,v in self._attr.items(): a += ',%s=%s' % (k,repr(v))
		return a+')'

	def append(self,obj):
		self._contents.append(obj)

	def __str__(self):
		return repr(self)

	def __getattr__(self,n):
		if self._attr.has_key(n): return self._attr[n]
		if n in self._contentNames: return self._contents[self._contentNames.index(n)]
		raise AttributeError(n)

	def _makePTagContents(self):
		return self._contents

	def makePTag(self):
		contents = self._makePTagContents()
		if contents is None: C = None
		else:
			C = []
			i = 0
			for c in contents:
				n = self._contentNames[i]
				i += 1
				if c is None: c = ''
				if isinstance(c,PseudoTag):
					t = c.makePTag()
				elif type(c) in (type(()), type([])):
					if n in self._noGroupNames:
						_makeSeqPTag(c,C)
						continue
					else:
						t = (n,None,_makeSeqPTag(c,[]),None)
				else:
					t = (n,None,[c],None)
				C.append(t)

		return (self._tagName,self._attr or None,C,None)

def namedPTag(tag,contents,**attr):
	attr.update({'_tagName': tag})
	return PseudoTag(contents is not None and list(contents) or None,**attr)

def _cPTag(y,tag):
	return namedPTag(tag,None,checked=getattr(y,tag) and "yes" or "no")

class Fund(PseudoTag):
	_tagName = 'fund'
	_contentNames = ['fundCode', 'fundName']

class APlanYear(PseudoTag):
	_noGroupNames = ('funds',)

class CurrentPlanYear(APlanYear):
	_tagName = 'currentPlanYear'
	_contentNames = ['accountYear', 'monthlySavingsPlan', 'payOutIncome', 'currentAccountType', 'funds']

class PreviousPlanYear(APlanYear):
	_tagName = 'previousPlanYear'
	_contentNames = ['accountYear', 'bundlesPlansTogether', 'payOutIncome', 'previousAccountType', 'funds']
					
class PlanYear(APlanYear):
	_tagName = 'planYear'
	_contentNames = ['accountYear', 'monthlySavingsPlan', 'accountType', 'currentPlanYear',
					'bundlesPlansTogether', 'payOutIncome', 'funds']

	def _error(self):
		if self.currentPlanYear:
			if self.accountYear[0]=='B' or self.bundlesPlansTogether: return 'Current Plan Year can\'t bundle!'
			if self.accountType.lower()=='PEP': return 'Current Plan year can\'t have PEP'
		else:
			if self.monthlySavingsPlan: return 'Previous Plan Years can\'t have monthlySavingsPlan'
		return None

	def _checkSame(self, monthlySavingsPlan, accountType, currentPlanYear,
				bundlesPlansTogether, payOutIncome):
		for a in 'monthlySavingsPlan', 'accountType', 'currentPlanYear', 'bundlesPlansTogether', 'payOutIncome':
			if locals()[a]!=getattr(self,a): return 'Different %s for same PlanYear' % a
		return None

class PlanManager(PseudoTag):
	_tagName = 'detailsOfPlanManager'
	_contentNames = ['nameOfManager', 'accountNumber', 'clientName']

class LetterOfAuthority(PseudoTag):
	_tagName = 'letterOfAuthority'
	_contentNames = ['loaFormCode', 'loaSerialNumber', 'planManager', 'planYears']
	_noGroupNames = ('planYears',)

class PersonalDetails(PseudoTag):
	_tagName = 'personalDetails'
	_contentNames = ['title', 'surname', 'firstNames', 'houseNumber', 'houseName', 'addressLine1', 'addressLine2', 'addressLine3', 'addressLine4',
							'postCode', 'telephoneNumber', 'emailAddress', 'fidelityClientNumber', 'existingAccountHolder']

	def _makePTagContents(self):
		contents = self._contents[:]
		for k in ('existingAccountHolder',):
			contents[self._contentNames.index(k)] = _cPTag(self,k)
		return contents

class LegalRequirement(PseudoTag):
	_tagName = 'legalRequirement'
	_contentNames = ['dateOfBirth', 'nationalInsuranceNumber','noNINumber', 'crownEmployeeSpouse']

	def _makePTagContents(self):
		contents = self._contents[:]
		for k in 'noNINumber', 'crownEmployeeSpouse':
			contents[self._contentNames.index(k)] = _cPTag(self,k)
		return contents

class DirectCreditMandate(PseudoTag):
	_tagName = 'directCreditMandate'
	_contentNames = ['bankName', 'bankAddressLine1', 'bankAddressLine2', 'bankAddressLine3', 'bankPostCode',
					'bankSortCode', 'bankAccountNumber', 'accountHolderName', 'bankCollectionAccountNumber']

class Rereg(PseudoTag):
	_tagName = 'rereg'
	_contentNames = ['formCode','sourceCode','serialNumber','personalDetails','legalRequirement','directCreditMandate','ifaDetails','lettersOfAuthority']
	_noGroupNames = ('lettersOfAuthority',)

class ReregEForm:
	dsn='REREGDB'
	singletons = ('existingAccountHolder', 'noNINumber', 'crownEmployeeSpouse')

	def __init__(self):
		I = self.execute_fetchall('''select formCode, sourceCode, serialNumber, loaFormCode, loaSerialNumber, channel
			from GenericInfo''')
		assert len(I)==1, "Only one row in GenericInfo"
		names = ['clientId']+PersonalDetails._contentNames+LegalRequirement._contentNames+DirectCreditMandate._contentNames
		self.formCode, self.sourceCode, self.serialNumber, self.loaFormCode, self.loaSerialNumber, self.channel = I[0]
		self._PDM = map(list,self.execute_fetchall('select '+','.join(names)+' from PersonalDetails'))
		err = []
		for V in self._PDM:
			f = lambda x, V=V, names=names: V[names.index(x)]
			for k in ('title', 'surname', 'firstNames', 'addressLine1',
					'postCode', 'telephoneNumber', 'existingAccountHolder',
					'dateOfBirth','noNINumber','crownEmployeeSpouse','bankName',
					'bankAddressLine1', 'bankPostCode', 'bankSortCode',
					'bankAccountNumber', 'accountHolderName',
					'bankCollectionAccountNumber') + (f('noNINumber')==0 and ('nationalInsuranceNumber',) or ()):
				v = f(k)
				if v is None:
					err.append("clientId=%s: %s is required!" % (V[0],k))
				else:
					if hasattr(v,'year'):
						V[names.index(k)] = "%4d%02d%02d" % (v.year,v.month,v.day)

		if err: raise ValueError('\n'.join(err))


		self._planManagerCache={}
		self._currentYears = []

	def connect(self):
		import mx.ODBC.Windows
		return mx.ODBC.Windows.connect(self.dsn)

	def execute_fetchall(self,statement):
		db = self.connect()
		cur = db.cursor()
		try:
			cur.execute(statement)
			L = cur.fetchall()
		except:
			del cur, db
			raise
		del cur, db
		return L

	def _incr(self,name):
		setattr(self,name,_incrNum(getattr(self,name)))

	def _popDetails(self,C,V):
		R = []
		for t in C._contentNames:
			v = V[0]
			del V[0]
			R.append(v)
		return C(R)

	def personalDetails(self,x):
		V = list(self._PDM[x])[1:]
		return [self._popDetails(PersonalDetails,V),
				self._popDetails(LegalRequirement,V),
				self._popDetails(DirectCreditMandate,V)]

	def clientId(self,x):
		return self._PDM[x][0]

	def numberOfClients(self):
		return len(self._PDM)

	def lettersOfAuthority(self,clientId):
		'''get all letters of authority for a given client id'''
		if type(clientId) is type(int):
			cids = str(clientId)
		else:
			cids = repr(clientId)
		I = self.execute_fetchall('''
SELECT Investments.planManagerId, Investments.accountYear,
Investments.monthlySavingsPlan, Investments.accountType, Investments.currentPlanYear, Investments.bundlesPlansTogether,
Investments.payOutIncome, Investments.fundCode,
FundCodesAndNames.FundName
FROM FundCodesAndNames INNER JOIN Investments ON FundCodesAndNames.FundCode = Investments.fundCode
WHERE Investments.clientId=%s
ORDER BY Investments.planManagerId, Investments.accountYear DESC
''' % cids)
		rn = ['planManagerId', 'accountYear', 'monthlySavingsPlan', 'accountType', 'currentPlanYear',
			'bundlesPlansTogether', 'payOutIncome', 'fundCode', 'FundName']
		P = {}
		for i in I:
			lookup = lambda x, rn=rn, i=i: i[rn.index(x)]
			pmi = i[0]
			if not P.has_key(pmi): P[pmi]={}
			Y = P[pmi]
			ay = i[1].strip()
			if ay.lower() in ['bundled', 'bundled isa', 'bundled pep']: ay = 'Bundled'
			elif not _validInt(ay,1900,2100): raise ValueError("bad account year '%s' for clientId = %s" % (ay,cids))
			monthlySavingsPlan, accountType, currentPlanYear,\
				bundlesPlansTogether, payOutIncome =\
						lookup('monthlySavingsPlan'), lookup('accountType'), \
						lookup('currentPlanYear'), lookup('bundlesPlansTogether'), \
						lookup('payOutIncome')
			inv = Fund((lookup('fundCode'), lookup('FundName')))
			if not Y.has_key(ay):
				Y[ay] = PlanYear([ay,monthlySavingsPlan, accountType, currentPlanYear, bundlesPlansTogether,
									payOutIncome]+[[inv]])
				err = Y[ay]._error()
			else:
				err = Y[ay]._checkSame(monthlySavingsPlan, accountType, currentPlanYear, bundlesPlansTogether,
									payOutIncome)
				Y[ay].funds.append(inv)
			if err: raise ValueError("error in year %s for clientId=%s\nerr=%s" % (ay,cids,err))
			P[pmi] = Y
		I = []
		for k, v in P.items():
			YK = v.keys()
			YK.sort(_yearCmp)
			Y = []
			c = 0
			for y in YK:
				y = v[y]
				if y.currentPlanYear:
					c += 1
	
					y = CurrentPlanYear([y.accountYear, _cPTag(y,'monthlySavingsPlan'),_cPTag(y,'payOutIncome'), namedPTag('currentAccountType',None,type=y.accountType)]
										+ [y.funds])
				else:
					y = PreviousPlanYear([y.accountYear, _cPTag(y,'bundlesPlansTogether'), _cPTag(y,'payOutIncome'), namedPTag('previousAccountType',None,type=y.accountType)]
										+ [y.funds])

				Y.append(y)
			if c:
				if c>1:
					raise ValueError("clientId=%s, more than one Current Year!" % cids)
				if not isinstance(Y[0],CurrentPlanYear):
					raise ValueError("clientId=%s, Current Year should be latest!" % cids)
				y = Y[0].accountYear
				if y not in self._currentYears: self._currentYears.append(y)

			I.append(LetterOfAuthority([self.loaFormCode,self.loaSerialNumber,self._planManager(clientId,k)]+[Y]))
			self._incr('loaSerialNumber')
		return I

	def Rereg(self,x):
		clientId = self.clientId(x)
		LOA = self.lettersOfAuthority(clientId)
		if not LOA: return None
		C = []
		a = C.append
		for t in 'formCode', 'sourceCode', 'serialNumber': a(getattr(self,t))
		C = C + self.personalDetails(x)+[None]+[LOA]
		rereg=Rereg(C,channel=self.channel)
		self._incr('serialNumber')
		return rereg

	def _planManager(self,clientId,planManagerId):
		if planManagerId not in self._planManagerCache.keys():
			if type(planManagerId) is type(''): ids = repr(planManagerId)
			else: ids = str(planManagerId)
			nameOfManager, accountNumber = self.execute_fetchall('''SELECT nameOfManager, accountNumber
					FROM DetailsOfPlanManager WHERE planManagerId=%s''' % ids)[0]
			self._planManagerCache[planManagerId] = (nameOfManager,accountNumber)
		else:
			nameOfManager, accountNumber = self._planManagerCache[planManagerId]

		if type(clientId) is type(0): cids = str(clientId)
		else: cids = repr(clientId)
		clientName = self.execute_fetchall('''SELECT clientName
					FROM PlanManagerClientNames WHERE clientId=%s AND planManagerId=%s''' % (cids,ids))[0]
		return PlanManager((nameOfManager,accountNumber,clientName))

def main():
	import pprint
	from rlextra.ers.framework import ParsedXMLProcessor
	F = ReregEForm()
	D = {
		'OUTDIR': 'eformout',
		'APPDIR': '.',
		'CGI': 0,
		'action': 'Pdf',
		'CONTEXT': 'text',
		}
	for x in xrange(F.numberOfClients()):
		print F.clientId(x),
		r = F.Rereg(x)
		if r:
			print
			D['outfile'] = 'dump%d.pdf' % x
			D['data'] = r.makePTag()
			ParsedXMLProcessor().go(D)
		else:
			print ' has no Inverstments in the table'

if __name__=='__main__':
	main()
