#copyright ReportLab Inc. 2001-2003
#see license.txt for license details
#history www.reportlab.co.uk/rl-cgi/viewcvs.cgi/rlextra/graphics/webcharts/chartcontroller.py
#$Header: /rl_home/repository/rlextra/graphics/webcharts/chartcontroller.py,v 1.11 2003/06/17 11:16:15 john Exp $
__version__=''' $Id: chartcontroller.py,v 1.11 2003/06/17 11:16:15 john Exp $ '''


from rlextra.ers.webapp import WebApp
from reportlab.lib.colors import toColor
from rlextra.graphics.quickchart import quickChart, getTestDrawing, parseDataBlock

class WebChartApp(WebApp):
    "Web test harness / demonstrator for our charts"
    def __init__(self):
        WebApp.__init__(self)
        self.viewDir = '.'
        self.defaultView = 'main'
        
    def testImage(self, request, response):
        myChart = getTestDrawing()
        bitmapBytes = myChart.asString('png')
        response.setContentType('image/png')
        response.write(bitmapBytes)

    def getLogo(self, request, response):
        "Return reportlab logo"
        bitmapBytes = open('corplogo.gif','rb').read()
        response.setContentType('image/png')
        response.write(bitmapBytes)
        
    def image(self, request, response):
        "Generate PNG chart according to input parameters"


        # extract any chart-related parameters, converting
        # from strings to correct data types.  Keep this
        # explicit instead of being clever and dynamic

        kw = {}

        kw['showBoundaries'] = request.params.has_key('showBoundaries')

        kw['chartType'] = request.params.get('chartType','bar')

        kw['seriesRelation'] = request.params.get('seriesRelation', 'sidebyside')

        kw['width'] = request.params.get('width','400')

        kw['height'] = request.params.get('height','300')

        if request.params.has_key('data'):
            kw['data'] = request.params['data']
        if request.params.has_key('categoryNames'):
            kw['categoryNames'] = request.params['categoryNames']
        if request.params.has_key('seriesNames'):
            kw['seriesNames'] = request.params['seriesNames']


        #chart colours
        if request.params.has_key('chartColors'):
            kw['chartColors'] = request.params['chartColors']
            chartColorText = request.params['chartColors']
            if chartColorText == 'None':
                chartColors = None
            else:
                chartColors = chartColorText
        else:
            chartColors = None
        kw['chartColors'] = chartColors

        #main title
        if request.params.has_key('titleText'):
            kw['titleText'] = request.params['titleText']
        if request.params.has_key('titleFontName'):
            kw['titleFontName'] = request.params['titleFontName']
        if request.params.has_key('titleFontSize'):
            kw['titleFontSize'] = request.params['titleFontSize']
        if request.params.has_key('titleFontColor'):
            kw['titleFontColor'] = request.params['titleFontColor']


        #x axis title
        if request.params.has_key('xTitleText'):
            kw['xTitleText'] = request.params['xTitleText']
        if request.params.has_key('xTitleFontName'):
            kw['xTitleFontName'] = request.params['xTitleFontName']
        if request.params.has_key('xTitleFontSize'):
            kw['xTitleFontSize'] = request.params['xTitleFontSize']
        if request.params.has_key('xTitleFontColor'):
            kw['xTitleFontColor'] = request.params['xTitleFontColor']

        #y axis title
        if request.params.has_key('yTitleText'):
            kw['yTitleText'] = request.params['yTitleText']
        if request.params.has_key('yTitleFontName'):
            kw['yTitleFontName'] = request.params['yTitleFontName']
        if request.params.has_key('yTitleFontSize'):
            kw['yTitleFontSize'] = request.params['yTitleFontSize']
        if request.params.has_key('yTitleFontColor'):
            kw['yTitleFontColor'] = request.params['yTitleFontColor']


        #x axis 
        kw['xAxisVisible'] = request.params.has_key('xAxisVisible')
        if request.params.has_key('xAxisFontName'):
            kw['xAxisFontName'] = request.params['xAxisFontName']
        if request.params.has_key('xAxisFontSize'):
            kw['xAxisFontSize'] = request.params['xAxisFontSize']
        if request.params.has_key('xAxisFontColor'):
            kw['xAxisFontColor'] = request.params['xAxisFontColor']

        #y axis 
        kw['yAxisVisible'] = request.params.has_key('yAxisVisible')
        if request.params.has_key('yAxisFontName'):
            kw['yAxisFontName'] = request.params['yAxisFontName']
        if request.params.has_key('yAxisFontSize'):
            kw['yAxisFontSize'] = request.params['yAxisFontSize']
        if request.params.has_key('yAxisFontColor'):
            kw['yAxisFontColor'] = request.params['yAxisFontColor']

        #data labels
        if request.params.has_key('dataLabelsType'):
            kw['dataLabelsType'] = request.params['dataLabelsType']
        if request.params.has_key('dataLabelsFontName'):
            kw['dataLabelsFontName'] = request.params['dataLabelsFontName']
        if request.params.has_key('dataLabelsFontSize'):
            kw['dataLabelsFontSize'] = request.params['dataLabelsFontSize']
        if request.params.has_key('dataLabelsFontColor'):
            kw['dataLabelsFontColor'] = request.params['dataLabelsFontColor']


        # plot area
        kw['xAxisGridLines'] = request.params.has_key('xAxisGridLines')
        kw['yAxisGridLines'] = request.params.has_key('yAxisGridLines')
        
        if request.params.has_key('plotColor'):
            kw['plotColor'] =  request.params['plotColor']
        else:
            kw['plotColor'] =  None
            


        # data labels
        kw['dataLabelsType'] = request.params.get('dataLabelsType', None)
        kw['dataLabelsFontName'] = request.params.get('dataLabelsFontName', None)
        kw['dataLabelsFontSize'] = request.params.get('dataLabelsFontSize', None)
        kw['dataLabelsFontColor'] = request.params.get('dataLabelsFontColor', None)


        #if they give us a background colour, use it.
        colorText = request.params.get('bgColor','None')
        if colorText == 'None':
            bgColor = None
        else:
            bgColor = toColor(colorText)
        kw['bgColor'] = bgColor

        #markers for lines eg, in scatter_lines_markers
        if request.params.has_key('markerType'):
            mk = request.params['markerType']
            if mk == 'None':
                mk = None
            kw['markerType'] = mk

        if request.params.has_key('markerSize'):
            ms = request.params['markerSize']
            if ms == 'None':
                ms = None
            else:
                ms = int(ms)
            kw['markerSize'] = ms

        #legend properties
        if request.params.has_key('legendPos'):
            lpos = request.params['legendPos']
            if lpos == 'None':
                lpos = None
            kw['legendPos'] = lpos

        if request.params.has_key('legendFontName'):
            lfont = request.params['legendFontName']
            if lfont == 'None':
                lfont = None
            kw['legendFontName'] = lfont

        if request.params.has_key('legendFontSize'):
            lsize = request.params['legendFontSize']
            if lsize == 'None':
                lsize = None
            kw['legendFontSize'] = lsize

        if request.params.has_key('legendFontColor'):
            lcol = request.params['legendFontColor']
            if lcol == 'None':
                lcol = None
            kw['legendFontColor'] = lcol


##        response.write("<pre>")
##        import pprint
##        pprint.pprint(kw, response)
##        response.write("</pre>")
##        return
        
        myChart = quickChart(**kw)
        bitmapBytes = myChart.asString('gif')
        response.setContentType('image/gif')
        response.write(bitmapBytes)



if __name__=='__main__':
    WebChartApp().handleCommandLine()
    
